require "player/Player"
require "player/SpriteAnimation"
require "camera"
--require "player/coin"

function love.load()
    love.graphics.setBackgroundColor(255, 255, 255)
	width = love.graphics.getWidth()
    height = love.graphics.getHeight()
    
    -- Load up the map
    loader = require("AdvTiledLoader.Loader")
    loader.path = "maps/"
    map = loader.load("map01.tmx")
    map:setDrawRange(0, 0, map.width * map.tileWidth, map.height * map.tileHeight)
    
    -- restrict the camera
    camera:setBounds(0, 0, map.width * map.tileWidth - width, map.height * map.tileHeight - height)
    
    p = Player:new()
    
    p.x = 300
    p.y = 300
    p.width = 32
    p.height = 32
    p.jumpSpeed = -800
    p.runSpeed = 500
    
    gravity = 1800
    hasJumped = false
    delay = 500
	
	console = false
    
    -- Load player animation
    playerSprites = SpriteAnimation:new("player/link.png", 16, 24, 6, 8)
    playerSprites:load(delay)
	playerSprites:switch(1, 1)
    
    --[[ Place random coins around the map
    math.randomseed(os.time())
    numCoins = 25
    coins = {}
    for i = 1, numCoins do
        local coinCollides = true
        while coinCollides do -- try to place a coin on a random spot around the map
            local coinX = math.random(1, map.width - 1) * map.tileWidth + map.tileWidth / 2
            local coinY = math.random(1, map.height - 1) * map.tileHeight + map.tileHeight / 2
            coins[i] = coin:new(coinX, coinY)
            
            -- if tile is occupied, try again
            coinCollides = coins[i]:isColliding(map)
        end
    end
    
    -- create coin animation
    coinSprites = SpriteAnimation:new("player/coin.png", 32, 32, 20, 1)
    coinSprites:load(delay)
    
    score = 0--]]
    gameState = "title"
    -- Load the options
    getOptions = love.filesystem.load("options.txt")
    options = getOptions()
	--changeSize(options[3] * options[2], options[4] * options[2])
	-- Load the title screen
	titleImage = love.graphics.newImage("title.png")
	titleImage:setFilter("nearest", "nearest")
	titleArrow = love.graphics.newImage("titlearrow.png")
	titleArrow:setFilter("nearest", "nearest")
	optionSelected = 1
	optionChosen = 1
	hasPressedEnter = false
	--[[ load the font of awesomeness ripped by Desgardes, will ask Crownjo
	    to rip font so I don't have to ask some random dude for permission--]]
	font = love.graphics.newImageFont("font.png", " abcdefghijklmnopqrstuvwxyz" ..
	    "ABCDEFGHIJKLMNOPQRSTUVWXYZ0" ..
		"123456789")
end

function love.update(dt)
    if gameState == "ingame" then
	playerSprites:start()
    -- check controls
    if love.keyboard.isDown("right") then
        p:moveRight()
        playerSprites:flip(false, false)
    end
    if love.keyboard.isDown("left") then
        p:moveLeft()
        playerSprites:flip(true, false)
    end
	if love.keyboard.isDown("up") then
	    p:moveUp()
		playerSprites:flip(false, false)	
	end
	if love.keyboard.isDown("down") then
	    p:moveDown()
        playerSprites:flip(false, false)		
	end
    --[[if love.keyboard.isDown("x") and not(hasJumped) then
        hasJumped = true
        p:jump()
    end--]]
 
    -- update the player's position and check for map collisions
    p:update(dt, gravity, map)
    
    -- update the sprite animation
	if p.face == "south" then
        if (p.state == "stand") then
            playerSprites:reset()
			playerSprites:switch(1, 1, 200)
        end
        if (p.state == "moveRight") or (p.state == "moveLeft") then
            playerSprites:switch(2, 8, 100)
        end
        --[[if (p.state == "jump") or (p.state == "fall") then
            playerSprites:reset()
            playerSprites:switch(3, 1, 300)
        end--]]
    end
	if p.face == "east" or p.face == "west" then
        if (p.state == "stand") then
            playerSprites:reset()
			playerSprites:switch(3, 1, 200)
        end
        if (p.state == "moveRight") or (p.state == "moveLeft") then
            playerSprites:switch(4, 7, 100)
        end
	end
	if p.face == "north" then
        if (p.state == "stand") then
            playerSprites:reset()
			playerSprites:switch(5, 1, 200)
        end
        if (p.state == "moveRight") or (p.state == "moveLeft") then
            playerSprites:switch(6, 8, 100)
        end
	end
    playerSprites:update(dt)
    
    --[[ update coin animations and check for player collisions
    for i in ipairs(coins) do
        coins[i]:update(dt)
        
        -- if player collides, add to score and remove coin
        if coins[i]:touchesObject(p) then
            score = score + 1
            table.remove(coins, i)
        end
    end--]]
    
    -- center the camera on the player
    camera:setPosition(math.floor(p.x - width / 2), math.floor(p.y - height / 2))
    end
end

function love.draw()
    love.graphics.setFont(font)
    if gameState == "ingame" then
    camera:set()
    
    -- round our x, y values
    local x, y = math.floor(p.x), math.floor(p.y)
    
    -- draw the map
    map:draw()
    
    --[[ draw coins
    for i in ipairs(coins) do
        coinSprites:start(coins[i].frame)
        coinSprites:draw(coins[i].x - coins[i].width / 2, coins[i].y - coins[i].height / 2)
    end--]]
    
    -- draw the player
    playerSprites:draw(x - p.width / 2, y - p.height / 2)

    camera:unset()
    
    --[[ draw the score
    love.graphics.print("Score: "..score, 700, 5)--]]
    
	if console == true then
    -- debug information
    local tileX = math.floor(p.x / map.tileWidth)
    local tileY = math.floor(p.y / map.tileHeight)
    
    love.graphics.setColor(255, 255, 255)
    love.graphics.print("Player coordinates: ("..x..","..y..")", 5, 5)
    love.graphics.print("Current state: "..p.state, 5, 20)
	love.graphics.print("Current facing direction: "..p.face, 5, 35)
    love.graphics.print("Current tile: ("..tileX..", "..tileY..")", 5, 50)
	end
    end
    if gameState == "title" then
        love.graphics.draw(titleImage, 0, 0)
		love.graphics.draw(titleArrow, 83, 143 + (17 * optionSelected) )
    end
	if gameState == "options" then
	    love.graphics.print("Video: ", 30, 20)
		love.graphics.print("1x", 75, 20)
		love.graphics.print("2x", 105, 20)
		love.graphics.print("Fullscreen", 135, 20)
	    love.graphics.print("Return to title", 30, 160)
		if optionSelected == 1 then
		    love.graphics.draw(titleArrow, 35 + 30 * optionChosen, 21)
		elseif optionSelected == 2 then
		    love.graphics.draw(titleArrow, 20, 160)
		end
	end
end

function love.keypressed(key, unicode)
    -- Control the title screen
   if gameState == "title" then
        if key == "return" and hasPressedEnter == false then
		    if optionSelected == 1 then
                gameState = "ingame"
			end
			if optionSelected == 3 then
			    gameState = "options"
				optionSelected = 1
			end
			hasPressedEnter = true
		end
		if key == "up" then
		    optionSelected = 1
		end
		if key == "down" then
		    optionSelected = 3
		end
   end
   if gameState == "options" then
       if key == "return" and hasPressedEnter == false then
	        if optionSelected == 2 then
			    gameState = "title"
		        hasPressedEnter = true
				optionSelected = 1
			end
	   end
       if key == "left" then
            if optionChosen > 1 then
                optionChosen = optionChosen - 1
		    end
       end
	   if key == "right" then
	        if optionChosen < 3 then
			    optionChosen = optionChosen + 1
			end
		end
		if key == "up" then
		    if optionSelected > 1 then
			    optionSelected = optionSelected - 1
			end
		end
		if key == "down" then
		    if optionSelected < 2 then
			    optionSelected = optionSelected + 1
			end
		end
   end
   if key == "f3" then
        console = not console
   end
end

function love.keyreleased(key)
    if key == "escape" then
        love.event.push("quit")   -- actually causes the app to quit
    end
    if (key == "right") or (key == "left") then
        p:stopX()
    end
	if (key == "up") or (key == "down") then
	    p:stopY()
	end
    if (key == "x") then
        hasJumped = false
    end
	if key == "return" then
	    hasPressedEnter = false
	end
end

function math.clamp(x, min, max)
  return x < min and min or (x > max and max or x)
end

function changeSize( width, height )
    success = love.graphics.setMode( width, height )
end