-- Player class
-- Author: Sean Laurvick
-- A simple Player object for use in the Platformer game.

Player = {}

-- Constructor
function Player:new()
    -- define our parameters here
    local object = {
    x = 0,
    y = 0,
    width = 0,
    height = 0,
    xSpeed = 0,
    ySpeed = 0,
    xSpeedMax = 800,
    ySpeedMax = 800,
    state = "",
	face = "south",
    --jumpSpeed = 0,
    runSpeed = 0--,
    --onFloor = false
    }
    setmetatable(object, { __index = Player })
    return object
end

-- Movement functions
--[[function Player:jump()
    if self.onFloor then
        self.ySpeed = self.jumpSpeed
        self.onFloor = false
    end
end--]]

function Player:moveRight()
    self.xSpeed = self.runSpeed
end

function Player:moveLeft()
    self.xSpeed = -1 * (self.runSpeed)
end

function Player:moveDown()
    self.ySpeed = self.runSpeed
end

function Player:moveUp()
    self.ySpeed = -1 * (self.runSpeed)
end

function Player:stopX()
    self.xSpeed = 0
end

function Player:stopY()
    self.ySpeed = 0
end

--[[ Do various things when the player hits a tile
function Player:collide(event)
    if event == "floor" then
        self.ySpeed = 0
        self.onFloor = true
    end
    if event == "ceiling" then
        self.ySpeed = 0
    end
end--]]

-- Update function
function Player:update(dt, gravity, map)
    local halfX = self.width / 2
    local halfY = self.height / 2
    
    --[[ apply gravity
    self.ySpeed = self.ySpeed + (gravity * dt)--]]
    
    -- limit the player's speed
    self.xSpeed = math.clamp(self.xSpeed, -self.xSpeedMax, self.xSpeedMax)
    self.ySpeed = math.clamp(self.ySpeed, -self.ySpeedMax, self.ySpeedMax)
    
    --[[ calculate vertical position and adjust if needed (OLD)
    local nextY = math.floor(self.y + (self.ySpeed * dt))
    if self.ySpeed < 0 then -- check upward
        if not(self:isColliding(map, self.x - halfX, nextY - halfY))
            and not(self:isColliding(map, self.x + halfX - 1, nextY - halfY)) then
            -- no collision, move normally
            self.y = nextY
            self.onFloor = false
        else
            -- collision, move to nearest tile border
            self.y = nextY + map.tileHeight - ((nextY - halfY) % map.tileHeight)
            self:collide("ceiling")
        end
    elseif self.ySpeed > 0 then -- check downward
        if not(self:isColliding(map, self.x - halfX, nextY + halfY))
            and not(self:isColliding(map, self.x + halfX - 1, nextY + halfY)) then
            -- no collision, move normally
            self.y = nextY
            self.onFloor = false
        else
            -- collision, move to nearest tile border
            self.y = nextY - ((nextY + halfY) % map.tileHeight)
            self:collide("floor")
        end
    end--]]

    -- calculate horizontal position and adjust if needed
    local nextX = math.floor(self.x + (self.xSpeed * dt))
    if self.xSpeed > 0 then -- check right
        if not(self:isColliding(map, nextX + halfX, self.y - halfY))
            and not(self:isColliding(map, nextX + halfX, self.y + halfY - 1)) then
            -- no collision
            self.x = nextX
        else
            -- collision, move to nearest tile
            self.x = nextX - ((nextX + halfX) % map.tileWidth)
        end
    elseif self.xSpeed < 0 then -- check left
        if not(self:isColliding(map, nextX - halfX, self.y - halfY))
            and not(self:isColliding(map, nextX - halfX, self.y + halfY - 1)) then
            -- no collision
            self.x = nextX
        else
            -- collision, move to nearest tile
            self.x = nextX + map.tileWidth - ((nextX - halfX) % map.tileWidth)
        end
    end
	
	-- calculate vertical position and adjust if needed
    local nextY = math.floor(self.y + (self.ySpeed * dt))
    if self.ySpeed > 0 then -- check down
        if not(self:isColliding(map, self.x - halfX, nextY + halfY))
            and not(self:isColliding(map, self.x + halfX - 1, nextY + halfY)) then
            -- no collision
            self.y = nextY
        else
            -- collision, move to nearest tile
            self.y = nextY - ((nextY + halfY) % map.tileHeight)
        end
    elseif self.ySpeed < 0 then -- check up
        if not(self:isColliding(map, self.x - halfX, nextY - halfY))
            and not(self:isColliding(map, self.x + halfX - 1, nextY - halfY)) then
            -- no collision
            self.y = nextY
        else
            -- collision, move to nearest tile
            self.y = nextY + map.tileHeight - ((nextY - halfY) % map.tileHeight)
        end
    end
    
    -- update the player's state
	self.state = self:getState()
	
	-- update facing direction
	if not(xSpeed == 0) and not(ySpeed == 0) then
        self.face = self:getFace()
	end
end

-- returns true if the coordinates given intersect a map tile
function Player:isColliding(map, x, y)
    -- get tile coordinates
    local layer = map.tl["Walls"]
    local tileX, tileY = math.floor(x / map.tileWidth), math.floor(y / map.tileHeight)
    
    -- grab the tile at given point
    local tile = layer.tileData(tileX, tileY)
    
    -- return true if the point overlaps a solid tile
    return not(tile == nil)
end

-- returns player's state as a string
function Player:getState()
    local myState = ""
    if self.xSpeed > 0 then
        myState = "moveRight"
    elseif self.xSpeed < 0 then
        myState = "moveLeft"
    else
        myState = "stand"
    end
    if self.ySpeed < 0 then
        myState = "moveUp"
    elseif self.ySpeed > 0 then
        myState = "moveDown"
    end
	if self.xSpeed == 0 and self.ySpeed == 0 then
	    myState = "stand"
	end
    return myState
end

-- returns facing direction as a string
function Player:getFace()
    local myFace = ""
	if self.ySpeed == 0 then
	    if self.xSpeed > 0 then
		    myFace = "east"
		elseif self.xSpeed < 0 then
		    myFace = "west"
		end
	end
	if self.xSpeed == 0 then
	    if self.ySpeed < 0 then
		    myFace = "north"
		elseif self.ySpeed > 0 then
		    myFace = "south"
		end
	end
	return myFace
end